@@ -28,6 +28,7 @@
 
 #include <config.h>
 
+#include <glib/gi18n.h>
 
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -281,9 +282,9 @@ animate_text (gpointer data)
 	gnome_canvas_item_move (ani_data->item->parent, 0.0, -10.0);
 
 	if (ani_data->item->parent->y1 <= 120.0) {
-		g_timeout_add (5 * 1000,
-			       display_introduction_message,
-			       ani_data->canvas);
+		g_timeout_add_seconds (5,
+				       display_introduction_message,
+				       ani_data->canvas);
 		g_free (ani_data);
 		return FALSE;
 	}
@@ -671,27 +672,26 @@ get_description_messages (xmlNodePtr node)
 {
 	xmlNodePtr paras;
 	GSList *list = NULL, *l;
-	const GList *langs = NULL;
+	const char * const * langs;
 	gint i;
 	gboolean started = FALSE;
 	char *best_value = NULL;
 
-	langs = gnome_i18n_get_language_list ("LC_MESSAGES");
+	langs = g_get_language_names ();
 	paras = node->children;
 
 	while (paras) {
 		while (paras) {
-			char *value = (char *)xmlNodeGetContent (paras);
+			xmlChar *value = xmlNodeGetContent (paras);
 			char *tmp;
 			xmlChar *cur, *best = NULL;
-			const GList *t;
 
 			if (paras->type == XML_ELEMENT_NODE &&
 					xmlStrEqual (paras->name, (const xmlChar *) "p") &&
 					value && value[0]) {
 				cur = xmlNodeGetLang (paras);
 
-				tmp = strip_newlines (value);
+				tmp = strip_newlines ((const gchar *) value);
 
 				if (!started) {
 					started = TRUE;
@@ -704,17 +704,19 @@ get_description_messages (xmlNodePtr node)
 					best_value = g_strdup (tmp);
 				}
 				else {
+					guint i;
+
 					if (!cur || xmlStrEqual (cur, (const xmlChar *) "C")) {
 						break;
 					}
 					/* See if the current lanaguage occurs
 					 * earlier than the previous best. */
-					for (t = langs; t; t = t->next) {
-						if (xmlStrEqual (t->data,
+					for (i = 0; langs[i] != NULL; ++i) {
+						if (xmlStrEqual ((const xmlChar *) langs[i],
 								best)) {
 							break;
 						}
-						else if (xmlStrEqual (t->data,
+						else if (xmlStrEqual ((const xmlChar *) langs[i],
 									cur)) {
 							xmlFree (best);
 							best = xmlStrdup (cur);
@@ -726,7 +728,7 @@ get_description_messages (xmlNodePtr node)
 							 * stop. We are not
 							 * going to go any
 							 * better. */
-							if (t == langs) {
+							if (i == 0) {
 								break;
 							}
 						}
@@ -781,46 +783,39 @@ create_date_string (const char *value)
 	return result;
 }
 
-static void
-display_version_info (GnomeCanvasGroup *group)
+static gboolean
+get_version_info (char **version_string,
+		  char **distributor_string,
+		  char **build_date_string)
 {
+	gchar *file;
+
 	xmlDocPtr about;
 	xmlNodePtr node;
 	xmlNodePtr bits;
 
-	gchar *file;
-
 	char *platform = NULL;
 	char *minor = NULL;
 	char *micro = NULL;
-	char *version_string = NULL;
-	char *distributor_string = NULL;
-	char *build_date_string = NULL;
-	char *text = NULL;
-
-	GnomeCanvasItem *info;
-	gdouble height = 0.0;
 
 	file = gnome_program_locate_file (NULL,
 					  GNOME_FILE_DOMAIN_APP_DATADIR,
 					  "gnome-version.xml",
 					  TRUE, NULL);
-	if (!file) {
-		show_error_dialog (_("Could not locate the file with GNOME version information."));
-		return;
-	}
+	if (!file)
+		return FALSE;
 
 	about = xmlParseFile (file);
 	g_free (file);
 
 	if (!about)
-		return;
+		return FALSE;
 
 	node = about->children;
 
 	if (g_ascii_strcasecmp ((const char *) node->name, "gnome-version")) {
 		xmlFreeDoc (about);
-		return;
+		return FALSE;
 	}
 
 	bits = node->children;
@@ -837,14 +832,14 @@ display_version_info (GnomeCanvasGroup *group)
 		if (!g_ascii_strcasecmp (name, "platform")
 		    && value && value[0])
 			platform = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "minor") && value && value[0])
+		else if (!g_ascii_strcasecmp (name, "minor") && value && value[0])
 			minor = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "micro") && value && value[0])
+		else if (!g_ascii_strcasecmp (name, "micro") && value && value[0])
 			micro = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "distributor") && value && value[0])
-			distributor_string = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "date") && value && value[0])
-			build_date_string = create_date_string (value);
+		else if (!g_ascii_strcasecmp (name, "distributor") && value && value[0])
+			*distributor_string = g_strdup (value);
+		else if (!g_ascii_strcasecmp (name, "date") && value && value[0])
+			*build_date_string = create_date_string (value);
 
 		bits = bits->next;
 		xmlFree (value);
@@ -853,29 +848,76 @@ display_version_info (GnomeCanvasGroup *group)
 	xmlFreeDoc (about);
 
 	if (!minor)
-		version_string = g_strconcat (platform, NULL);
+		*version_string = g_strdup (platform);
 
-	if (!version_string && !micro)
-		version_string = g_strconcat (platform, ".", minor, NULL);
+	if (!*version_string && !micro)
+		*version_string = g_strconcat (platform, ".", minor, NULL);
 
-	if (!version_string)
-		version_string = g_strconcat (platform, ".", minor, ".",
-					      micro, NULL);
+	if (!*version_string)
+		*version_string = g_strconcat (platform, ".", minor, ".",
+					       micro, NULL);
 
 	g_free (platform);
 	g_free (minor);
 	g_free (micro);
 
+	return TRUE;
+}
+
+static gboolean
+display_version_info_on_term (void)
+{
+	char *version_string = NULL;
+	char *distributor_string = NULL;
+	char *build_date_string = NULL;
+
+	if (!get_version_info (&version_string,
+			       &distributor_string,
+			       &build_date_string)) {
+		g_printerr (_("Could not get information about GNOME version."));
+		return FALSE;
+	}
+
+	g_print (_("%s: %s\n"), _("Version"), version_string);
+	g_print (_("%s: %s\n"), _("Distributor"), distributor_string);
+	g_print (_("%s: %s\n"), _("Build Date"), build_date_string);
+		 
+	g_free (version_string);
+	g_free (distributor_string);
+	g_free (build_date_string);
+
+	return TRUE;
+}
+
+static void
+display_version_info (GnomeCanvasGroup *group)
+{
+	char *version_string = NULL;
+	char *distributor_string = NULL;
+	char *build_date_string = NULL;
+	char *format = NULL;
+	char *text = NULL;
+
+	GnomeCanvasItem *info;
+	gdouble height = 0.0;
+
+	if (!get_version_info (&version_string,
+			       &distributor_string,
+			       &build_date_string))
+		show_error_dialog (_("Could not get information about GNOME version."));
+
 	info = gnome_canvas_item_new (group,
 				      gnome_canvas_group_get_type (),
 				      "x", 10.0,
 				      NULL);
 
+	format = g_strdup_printf ("<b>%s</b>%%s", _("%s: "));
+
 	if (version_string && version_string[0]) {
 		gdouble tmp;
 		GnomeCanvasItem *item;
 
-		text = g_strdup_printf ("<b>%s: </b>%s",
+		text = g_strdup_printf (format,
 					_("Version"), version_string);
 		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (info),
 					      gnome_canvas_text_get_type (),
@@ -894,7 +936,7 @@ display_version_info (GnomeCanvasGroup *group)
 		gdouble tmp;
 		GnomeCanvasItem *item;
 
-		text = g_strdup_printf ("<b>%s: </b>%s",
+		text = g_strdup_printf (format,
 					_("Distributor"), distributor_string);
 		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (info),
 					      gnome_canvas_text_get_type (),
@@ -913,7 +955,7 @@ display_version_info (GnomeCanvasGroup *group)
 		gdouble tmp;
 		GnomeCanvasItem *item;
 
-		text = g_strdup_printf ("<b>%s: </b>%s",
+		text = g_strdup_printf (format,
 					_("Build Date"), build_date_string);
 		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (info),
 					      gnome_canvas_text_get_type (),
@@ -928,6 +970,7 @@ display_version_info (GnomeCanvasGroup *group)
 		height += tmp + 4.0;
 	}
 
+	g_free (format);
 	g_free (version_string);
 	g_free (distributor_string);
 	g_free (build_date_string);
@@ -1207,11 +1250,18 @@ generate_randomness (void)
 	g_rand_free (generator);
 }
 		
+static gboolean gnome_version = FALSE;
+
+static const GOptionEntry options[] = {
+  { "gnome-version", 0, 0, G_OPTION_ARG_NONE, &gnome_version, N_("Display information on this GNOME version"), NULL },
+  { NULL }
+};
 
 /* main */
 int
 main (int argc, char **argv)
 {
+	GOptionContext *context;
 	GnomeProgram *program;
 	GtkWidget *dialog;
 
@@ -1219,13 +1269,23 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	program = gnome_program_init ("gnome-about", VERSION,
 				      LIBGNOMEUI_MODULE,
 				      argc, argv,
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      NULL);
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR
-			"/gnome-logo-icon-transparent.png");
+
+	if (gnome_version) {
+		g_object_unref (program);
+		return (display_version_info_on_term ()) ? 0 : 1;
+	}
+
+	gtk_window_set_default_icon_from_file (GNOME_ICONDIR
+			"/gnome-logo-icon-transparent.png", NULL);
 
 	dialog = create_about_dialog ();
 	if (!dialog)
